# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: storage_agent.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'storage_agent.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13storage_agent.proto\"\x8c\x01\n\x10\x43ompoundBundleId\x12\x12\n\nsource_eid\x18\x01 \x01(\t\x12\x1a\n\x12\x63reation_timestamp\x18\x03 \x01(\x04\x12\x17\n\x0fsequence_number\x18\x04 \x01(\x04\x12\x17\n\x0f\x66ragment_offset\x18\x05 \x01(\x04\x12\x16\n\x0epayload_length\x18\x06 \x01(\x04\":\n\x14\x42undleMetadataFilter\x12\x15\n\x08\x65id_glob\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0b\n\t_eid_glob\"\x90\x01\n\x0bStorageCall\x12$\n\toperation\x18\x01 \x01(\x0e\x32\x11.StorageOperation\x12\x1f\n\x02id\x18\x02 \x01(\x0b\x32\x11.CompoundBundleIdH\x00\x12)\n\x08metadata\x18\x04 \x01(\x0b\x32\x15.BundleMetadataFilterH\x00\x42\x0f\n\rbundle_filter*\xa3\x01\n\x10StorageOperation\x12!\n\x1dSTORAGE_OPERATION_UNSPECIFIED\x10\x00\x12\"\n\x1eSTORAGE_OPERATION_LIST_BUNDLES\x10\x01\x12$\n STORAGE_OPERATION_DELETE_BUNDLES\x10\x02\x12\"\n\x1eSTORAGE_OPERATION_PUSH_BUNDLES\x10\x03\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'storage_agent_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_STORAGEOPERATION']._serialized_start=374
  _globals['_STORAGEOPERATION']._serialized_end=537
  _globals['_COMPOUNDBUNDLEID']._serialized_start=24
  _globals['_COMPOUNDBUNDLEID']._serialized_end=164
  _globals['_BUNDLEMETADATAFILTER']._serialized_start=166
  _globals['_BUNDLEMETADATAFILTER']._serialized_end=224
  _globals['_STORAGECALL']._serialized_start=227
  _globals['_STORAGECALL']._serialized_end=371
# @@protoc_insertion_point(module_scope)
