# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: aap2.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'aap2.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\naap2.proto\x12\x04\x61\x61p2\"\xf0\x01\n\nAAPMessage\x12 \n\x07welcome\x18\x01 \x01(\x0b\x32\r.aap2.WelcomeH\x00\x12(\n\x06\x63onfig\x18\x02 \x01(\x0b\x32\x16.aap2.ConnectionConfigH\x00\x12\x1e\n\x03\x61\x64u\x18\x03 \x01(\x0b\x32\x0f.aap2.BundleADUH\x00\x12-\n\x0e\x64ispatch_event\x18\x04 \x01(\x0b\x32\x13.aap2.DispatchEventH\x00\x12\x1a\n\x04link\x18\x05 \x01(\x0b\x32\n.aap2.LinkH\x00\x12$\n\tkeepalive\x18\x06 \x01(\x0b\x32\x0f.aap2.KeepaliveH\x00\x42\x05\n\x03msg\"\x1a\n\x07Welcome\x12\x0f\n\x07node_id\x18\x01 \x01(\t\"\x8c\x01\n\x10\x43onnectionConfig\x12\x15\n\ris_subscriber\x18\x01 \x01(\x08\x12!\n\tauth_type\x18\x02 \x01(\x0e\x32\x0e.aap2.AuthType\x12\x0e\n\x06secret\x18\x03 \x01(\t\x12\x13\n\x0b\x65ndpoint_id\x18\x04 \x01(\t\x12\x19\n\x11keepalive_seconds\x18\x05 \x01(\r\"\xbd\x01\n\tBundleADU\x12\x0f\n\x07src_eid\x18\x01 \x01(\t\x12\x0f\n\x07\x64st_eid\x18\x02 \x01(\t\x12\x15\n\rreport_to_eid\x18\x07 \x01(\t\x12\x1d\n\x15\x63reation_timestamp_ms\x18\x03 \x01(\x04\x12\x17\n\x0fsequence_number\x18\x04 \x01(\x04\x12\x16\n\x0epayload_length\x18\x05 \x01(\x04\x12\'\n\tadu_flags\x18\x06 \x03(\x0e\x32\x14.aap2.BundleADUFlags\"\xc2\x01\n\x06\x42undle\x12\x0f\n\x07src_eid\x18\x01 \x01(\t\x12\x0f\n\x07\x64st_eid\x18\x02 \x01(\t\x12\x1d\n\x15\x63reation_timestamp_ms\x18\x03 \x01(\x04\x12\x17\n\x0fsequence_number\x18\x04 \x01(\x04\x12\x16\n\x0epayload_length\x18\x05 \x01(\x04\x12\x17\n\x0f\x66ragment_offset\x18\x06 \x01(\x04\x12\x18\n\x10total_adu_length\x18\x07 \x01(\x04\x12\x13\n\x0blifetime_ms\x18\x08 \x01(\x04\"\xfa\x01\n\x12\x42undleDispatchInfo\x12\x17\n\x0fserialized_size\x18\x01 \x01(\x04\x12\x1b\n\x13min_frag_size_first\x18\x02 \x01(\x04\x12\x1a\n\x12min_frag_size_last\x18\x03 \x01(\x04\x12\x18\n\x10\x64ispatch_time_ms\x18\x04 \x01(\x04\x12\x1a\n\x12\x65xpiration_time_ms\x18\x05 \x01(\x04\x12\x1d\n\x15\x64ispatched_to_node_id\x18\x06 \x01(\t\x12\x1e\n\x16\x64ispatched_to_cla_addr\x18\x07 \x01(\t\x12\x1d\n\x15max_bundle_size_bytes\x18\x08 \x01(\x04\"\x8d\x01\n\rDispatchEvent\x12\x1c\n\x06\x62undle\x18\x01 \x01(\x0b\x32\x0c.aap2.Bundle\x12$\n\x06reason\x18\x02 \x01(\x0e\x32\x14.aap2.DispatchReason\x12\x38\n\x16\x61\x64\x64itional_information\x18\x03 \x01(\x0b\x32\x18.aap2.BundleDispatchInfo\"t\n\x04Link\x12 \n\x06status\x18\x01 \x01(\x0e\x32\x10.aap2.LinkStatus\x12\x14\n\x0cpeer_node_id\x18\x02 \x01(\t\x12\x15\n\rpeer_cla_addr\x18\x03 \x01(\t\x12\x1d\n\x04\x66lag\x18\x04 \x01(\x0e\x32\x0f.aap2.LinkFlags\"\x0b\n\tKeepalive\"\x91\x01\n\x0b\x41\x41PResponse\x12-\n\x0fresponse_status\x18\x01 \x01(\x0e\x32\x14.aap2.ResponseStatus\x12-\n\x0f\x64ispatch_result\x18\x02 \x01(\x0b\x32\x14.aap2.DispatchResult\x12$\n\x0e\x62undle_headers\x18\x03 \x01(\x0b\x32\x0c.aap2.Bundle\"\x99\x01\n\x0e\x44ispatchResult\x12\x34\n\tnext_hops\x18\x01 \x03(\x0b\x32!.aap2.DispatchResult.NextHopEntry\x1aQ\n\x0cNextHopEntry\x12\x0f\n\x07node_id\x18\x01 \x01(\t\x12\x17\n\x0f\x66ragment_offset\x18\x02 \x01(\x04\x12\x17\n\x0f\x66ragment_length\x18\x03 \x01(\x04*{\n\x08\x41uthType\x12\x15\n\x11\x41UTH_TYPE_DEFAULT\x10\x00\x12\x19\n\x15\x41UTH_TYPE_FIB_CONTROL\x10\x01\x12\x1d\n\x19\x41UTH_TYPE_BUNDLE_DISPATCH\x10\x02\x12\x1e\n\x1a\x41UTH_TYPE_FIB_AND_DISPATCH\x10\x03*x\n\x0e\x42undleADUFlags\x12\x15\n\x11\x42UNDLE_ADU_NORMAL\x10\x00\x12\x13\n\x0f\x42UNDLE_ADU_BPDU\x10\x01\x12\x1c\n\x18\x42UNDLE_ADU_WITH_BDM_AUTH\x10\x02\x12\x1c\n\x18\x42UNDLE_ADU_STATUS_REPORT\x10\x03*\xdf\x01\n\x0e\x44ispatchReason\x12\x1f\n\x1b\x44ISPATCH_REASON_UNSPECIFIED\x10\x00\x12 \n\x1c\x44ISPATCH_REASON_NO_FIB_ENTRY\x10\x01\x12!\n\x1d\x44ISPATCH_REASON_LINK_INACTIVE\x10\x02\x12&\n\"DISPATCH_REASON_CLA_ENQUEUE_FAILED\x10\x03\x12\x1d\n\x19\x44ISPATCH_REASON_TX_FAILED\x10\x04\x12 \n\x1c\x44ISPATCH_REASON_TX_SUCCEEDED\x10\x05*u\n\nLinkStatus\x12\x1b\n\x17LINK_STATUS_UNSPECIFIED\x10\x00\x12\x12\n\x0eLINK_STATUS_UP\x10\x03\x12\x14\n\x10LINK_STATUS_DOWN\x10\x04\x12 \n\x1cLINK_STATUS_DROP_CLA_MAPPING\x10\x05*5\n\tLinkFlags\x12\x12\n\x0eLINK_FLAG_NONE\x10\x00\x12\x14\n\x10LINK_FLAG_DIRECT\x10\x01*\x85\x02\n\x0eResponseStatus\x12\x1f\n\x1bRESPONSE_STATUS_UNSPECIFIED\x10\x00\x12\x1b\n\x17RESPONSE_STATUS_SUCCESS\x10\x01\x12\x17\n\x13RESPONSE_STATUS_ACK\x10\x02\x12\x19\n\x15RESPONSE_STATUS_ERROR\x10\x08\x12\x1b\n\x17RESPONSE_STATUS_TIMEOUT\x10\t\x12#\n\x1fRESPONSE_STATUS_INVALID_REQUEST\x10\n\x12\x1d\n\x19RESPONSE_STATUS_NOT_FOUND\x10\x0b\x12 \n\x1cRESPONSE_STATUS_UNAUTHORIZED\x10\x0c\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'aap2_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_AUTHTYPE']._serialized_start=1655
  _globals['_AUTHTYPE']._serialized_end=1778
  _globals['_BUNDLEADUFLAGS']._serialized_start=1780
  _globals['_BUNDLEADUFLAGS']._serialized_end=1900
  _globals['_DISPATCHREASON']._serialized_start=1903
  _globals['_DISPATCHREASON']._serialized_end=2126
  _globals['_LINKSTATUS']._serialized_start=2128
  _globals['_LINKSTATUS']._serialized_end=2245
  _globals['_LINKFLAGS']._serialized_start=2247
  _globals['_LINKFLAGS']._serialized_end=2300
  _globals['_RESPONSESTATUS']._serialized_start=2303
  _globals['_RESPONSESTATUS']._serialized_end=2564
  _globals['_AAPMESSAGE']._serialized_start=21
  _globals['_AAPMESSAGE']._serialized_end=261
  _globals['_WELCOME']._serialized_start=263
  _globals['_WELCOME']._serialized_end=289
  _globals['_CONNECTIONCONFIG']._serialized_start=292
  _globals['_CONNECTIONCONFIG']._serialized_end=432
  _globals['_BUNDLEADU']._serialized_start=435
  _globals['_BUNDLEADU']._serialized_end=624
  _globals['_BUNDLE']._serialized_start=627
  _globals['_BUNDLE']._serialized_end=821
  _globals['_BUNDLEDISPATCHINFO']._serialized_start=824
  _globals['_BUNDLEDISPATCHINFO']._serialized_end=1074
  _globals['_DISPATCHEVENT']._serialized_start=1077
  _globals['_DISPATCHEVENT']._serialized_end=1218
  _globals['_LINK']._serialized_start=1220
  _globals['_LINK']._serialized_end=1336
  _globals['_KEEPALIVE']._serialized_start=1338
  _globals['_KEEPALIVE']._serialized_end=1349
  _globals['_AAPRESPONSE']._serialized_start=1352
  _globals['_AAPRESPONSE']._serialized_end=1497
  _globals['_DISPATCHRESULT']._serialized_start=1500
  _globals['_DISPATCHRESULT']._serialized_end=1653
  _globals['_DISPATCHRESULT_NEXTHOPENTRY']._serialized_start=1572
  _globals['_DISPATCHRESULT_NEXTHOPENTRY']._serialized_end=1653
# @@protoc_insertion_point(module_scope)
